package com.example.bpa.app;

import android.app.Application;
import android.util.Log;


import com.example.bpa.Model.IMessageLogNotify;
import com.example.bpa.config.ConfigName;
import com.example.bpa.db.QueryDB;
import com.example.bpa.db.mode.BPA_LOG;
import com.example.bpa.db.mode.BPA_MATERIAL;
import com.example.bpa.db.mode.BPA_SILOS;
import com.example.bpa.db.mode.BPA_SILOSANDMATERIAL;
import com.example.bpa.db.mode.BPA_SUGAR;
import com.example.bpa.db.mode.BPA_USER;
import com.example.bpa.helper.Json;
import com.example.bpa.helper.MessageLog;
import com.example.bpa.helper.SdCart;
import com.example.bpa.helper.ToastUtil;

import java.util.ArrayList;

/**
 * 程序启动执行
 * 创建:fengyoufu 20230413
 */
public class ICSApp extends Application {
    private static ICSApp instance;

    @Override
    public void onCreate() {
        super.onCreate();


        MessageLog.MsgNotify = new IMessageLogNotify() {
            @Override
            public void ErrorMsg(String msg) {
                Log.e("Error", msg);

            }

            @Override
            public void InfoMsg(String msg, String datetime) {
                Log.i("Info", msg);
            }

            @Override
            public void WarnMsg(String msg) {
                Log.w("Warn", msg);
            }
        };


        //1.设置程序active,初始化Main函数进程
        ConfigName.getInstance().dishesCon = this;
        ToastUtil.init(this);

        //2.初始化SD卡,数据库DB
        SdCart.getInstance().initSD();
        BusinessServer.get().Start();
        //3.初始化数据
        InitDBdata();
        //4.初始化Main
        Main.getInstance();
    }

    /**
     * 初始化数据库数据(323232)
     */
    public void InitDBdata() {
        //1.没有用户数据 默认写一个admin
        if (QueryDB.GetUserALL().size() <= 0) {
            BPA_USER user = new BPA_USER();
            user.id = "20230415-fyf";
            user.name = "管理员";
            user.account = "admin";
            user.pass = "123456";
            user.deviceID = ConfigName.getInstance().DeviceId;
            user.userID = "20230415-fyf";
            user.exp = "管理员";
            QueryDB.AddUser(user);
            //10个料仓
            if (QueryDB.GetSilosALL().size() == 0) {
                for (int i = 0; i < 10; i++) {
                    BPA_SILOS silo = new BPA_SILOS();
                    silo.id = java.util.UUID.randomUUID().toString();
                    silo.num = i + 1;
                    BPA_SILOSANDMATERIAL sm = new BPA_SILOSANDMATERIAL();
                    sm.silosID = silo.id;
                    sm.materialID = "";
                    QueryDB.AddSilos(silo);
                    QueryDB.AddSilosAndMaterial(sm);
                }
            }

            if (QueryDB.GetSugarALL().size() == 0) {
                BPA_SUGAR sugar_A_1 = new BPA_SUGAR();
                BPA_SUGAR sugar_A_5 = new BPA_SUGAR();
                BPA_SUGAR sugar_A_9 = new BPA_SUGAR();
                BPA_SUGAR sugar_A_2 = new BPA_SUGAR();
                BPA_SUGAR sugar_B_1 = new BPA_SUGAR();
                BPA_SUGAR sugar_B_5 = new BPA_SUGAR();
                BPA_SUGAR sugar_B_9 = new BPA_SUGAR();
                BPA_SUGAR sugar_B_2 = new BPA_SUGAR();
                sugar_A_1.sugarName = "果糖A_1";
                sugar_A_5.sugarName = "果糖A_5";
                sugar_A_9.sugarName = "果糖A_9";
                sugar_A_2.sugarName = "果糖A_2";
                sugar_B_1.sugarName = "果糖B_1";
                sugar_B_5.sugarName = "果糖B_5";
                sugar_B_9.sugarName = "果糖B_9";
                sugar_B_2.sugarName = "果糖B_2";
                sugar_A_1.plcvar = "M9.0";
                sugar_A_5.plcvar = "M9.1";
                sugar_A_9.plcvar = "M9.2";
                sugar_A_2.plcvar = "M9.3";
                sugar_B_1.plcvar = "M9.4";
                sugar_B_5.plcvar = "M9.5";
                sugar_B_9.plcvar = "M9.6";
                sugar_B_2.plcvar = "M9.7";
                sugar_A_1.plcvar = "M9.8";
                QueryDB.AddSugar(sugar_A_1);
                QueryDB.AddSugar(sugar_A_5);
                QueryDB.AddSugar(sugar_A_9);
                QueryDB.AddSugar(sugar_A_2);
                QueryDB.AddSugar(sugar_B_1);
                QueryDB.AddSugar(sugar_B_5);
                QueryDB.AddSugar(sugar_B_9);
                QueryDB.AddSugar(sugar_B_2);
            }
        }
    }

    public void AddTestLC() {
        BPA_SILOS par = new BPA_SILOS();
        par.id = "1231333";
        par.num = 1;
        par.name = "料仓";
        par.warningValue = 49;
        par.thrsoleValue = 20;
        par.plcValue = "232";
        par.bValue = 30;
        par.jValue = 23;
        par.status = 1;
        par.Sort = 0;
        QueryDB.AddSilos(par);

        //2.获取物料
        ArrayList<BPA_MATERIAL> materials = QueryDB.GetMaterialALL();

        BPA_SILOSANDMATERIAL al = new BPA_SILOSANDMATERIAL();
        al.silosID = "1231333";
        al.materialID = materials.get(0).id;
        QueryDB.AddSilosAndMaterial(al);
    }

    /**
     * 测试数据库 新增 修改 删除
     */
    public void Test() {
        //新增
        BPA_MATERIAL ba = new BPA_MATERIAL();
        ba.deviceID = java.util.UUID.randomUUID().toString();
        ba.userID = "张三---------";
        ba.exp = "测试-----";
        ba.name = "测试物料12312----";
        ba.imgUrl = "wwwwwwwww---";
        QueryDB.AddMaterial(ba);

        //查询
        ArrayList<BPA_MATERIAL> wl = QueryDB.GetMaterialALL();
        for (BPA_MATERIAL d : wl
        ) {
            String json = new Json<BPA_MATERIAL>().objectToJson(BPA_MATERIAL.class, d);
            Log.i("日志", "物料查询:" + json);

            //d.name="测试修改";
//			QueryDB.UpdateMaterial(d);
        }

        //删除
        //808be95d-68ae-4c7c-8e9e-b064408a61db
//		BPA_MATERIAL ba1=new BPA_MATERIAL();
//		ba1.id="fa3e7c53-d349-4637-9d97-4ba96b8b1d65";
//		QueryDB.DeleteMaterial(ba1);


    }
}
